import {BehaviorSubject, filter, interval, Subscription, switchMap} from "rxjs";
import {Injectable} from "@angular/core";
import {AuthService} from "@service/auth.service";
import {environment} from "@environment";
import ApiService from "@api/api.service";

@Injectable({
  providedIn: 'root',
})
export class TokenRefreshService {
  private tokenRefreshSubscription: Subscription | undefined;
  private tokenRefreshing$ = new BehaviorSubject<boolean>(false);
  private refreshTokenExpireMs: number = environment.retryDelay;

  constructor(private authService: AuthService) {
    this.setRefreshTokenExpireMs(AuthService.tokenExpiresIn.getTime() - 1000 - Date.now());

    authService.expireTokenChange.subscribe(date => {
      console.debug('Expire token change event received:', date);
      this.setRefreshTokenExpireMs(date.getTime() - 1000 - Date.now());
    });
  }

  public startTokenRefresh(date: Date | null = null): void {
    if (date)
      this.refreshTokenExpireMs = new Date(date).getTime() - 1000 - Date.now();

    console.debug(this.tokenRefreshSubscription);
    if (this.tokenRefreshSubscription && !this.tokenRefreshSubscription.closed)
      return;

    this.tokenRefreshSubscription = interval(this.refreshTokenExpireMs).pipe(
      filter(isRefreshing => !isRefreshing),
      switchMap(() => {
        this.tokenRefreshing$.next(true);
        console.debug('Send query to refresh token');
        return this.authService.refreshToken();
      })
    ).subscribe({
      next: (_) => {
        this.tokenRefreshing$.next(false);
      },
      error: error => {
        this.tokenRefreshing$.next(false);
        localStorage.removeItem(ApiService.tokenKey);
      }
    });
  }

  public getTokenRefreshing$(): BehaviorSubject<boolean> {
    return this.tokenRefreshing$;
  }

  public stopTokenRefresh(): void {
    if (this.tokenRefreshSubscription && !this.tokenRefreshSubscription.closed) {
      this.tokenRefreshSubscription.unsubscribe();
      this.tokenRefreshSubscription = undefined;
    }
  }

  public setRefreshTokenExpireMs(expireMs: number): void {
    if (expireMs < environment.retryDelay)
      expireMs = environment.retryDelay;

    this.refreshTokenExpireMs = expireMs;

    console.log(expireMs);
    this.stopTokenRefresh();
    this.startTokenRefresh();
  }
}
